from acex.configuration.components.base_component import ConfigComponent
from acex.configuration.components.interfaces import Interface
from acex.configuration.components.network_instances import L3Vrf
from acex_devkit.models.composed_configuration import (
    SnmpConfig, 
    SnmpUser as SnmpUserAttributes,
    SnmpServer as SnmpServerAttributes,
    TrapEvent,
    SnmpCommunity as SnmpCommunityAttributes,
    ReferenceTo, 
    ReferenceFrom
)

class SnmpGlobal(ConfigComponent):
    type = "snmp"
    model_cls = SnmpConfig

class SnmpUser(ConfigComponent):
    type = "snmp_user"
    model_cls = SnmpUserAttributes

class SnmpServer(ConfigComponent):
    type = "snmp_trap_server"
    model_cls = SnmpServerAttributes

    #def _add_vrf(self):
    #    if self.kwargs.get('network_instance'):
    #        ni = self.kwargs.pop("network_instance")
    #        if isinstance(ni, type(None)):
    #            pass
    #        elif isinstance(ni, str):
    #            ref = ReferenceFrom(pointer=f"network_instances.{ni}")
    #            self.kwargs["network_instance"] = ref
#
    #        elif isinstance(ni, L3Vrf):
    #            ref = ReferenceFrom(pointer=f"network_instances.{ni.name}")
    #            self.kwargs["network_instance"] = ref

    def pre_init(self):
        #self._add_vrf()
        # Resolve source_interface
        if "source_interface" in self.kwargs:
            si = self.kwargs.pop("source_interface")
            if isinstance(si, type(None)):
                pass
            elif isinstance(si, str):
                ref = ReferenceTo(pointer=f"interfaces.{si}")
                self.kwargs["source_interface"] = ref

            elif isinstance(si, Interface):
                ref = ReferenceTo(pointer=f"interfaces.{si.name}")
                self.kwargs["source_interface"] = ref

class SnmpTrap(ConfigComponent):
    type = "snmp_trap"
    model_cls = TrapEvent

class SnmpCommunity(ConfigComponent):
    type = "snmp_community"
    model_cls = SnmpCommunityAttributes

    def pre_init(self):
        # Resolve source_interface
        if "source_interface" in self.kwargs:
            si = self.kwargs.pop("source_interface")
            if isinstance(si, type(None)):
                pass
            elif isinstance(si, str):
                ref = ReferenceTo(pointer=f"interfaces.{si}")
                self.kwargs["source_interface"] = ref

            elif isinstance(si, Interface):
                ref = ReferenceTo(pointer=f"interfaces.{si.name}")
                self.kwargs["source_interface"] = ref




from .adapter_base import AdapterBase
from .datasource_adapter import DatasourcePluginAdapter

from .asset_adapter import AssetAdapter
from .logical_node_adapter import LogicalNodeAdapter
from .node_adapter import NodeAdapter
from .site_adapter import SiteAdapter
from .contact_adapter import ContactAdapter


"""Inventory management for assets, nodes, and logical nodes."""

from acex.inventory.inventory import Inventory
from acex.inventory.node_service import NodeService
from acex.inventory.logical_node_service import LogicalNodeService
from acex.inventory.site_service import SiteService
from acex.inventory.contact_service import ContactService

__all__ = [
    "Inventory",
    "NodeService",
    "LogicalNodeService",
    "SiteService",
    "ContactService",
]



from acex.configuration.components.base_component import ConfigComponent
from acex_devkit.models.composed_configuration import VTPAttributes

class Vtp(ConfigComponent):
    type = "vtp"
    model_cls = VTPAttributes


import inspect
import functools
from fastapi import APIRouter
from acex.constants import BASE_URL
from acex.models import LogicalNode



def get_response_model(func):
    sig = inspect.signature(func)
    return_annotation = sig.return_annotation
    if return_annotation is inspect.Signature.empty:
        return None
    return return_annotation



def create_router(automation_engine):

    router = APIRouter(prefix=f"{BASE_URL}/inventory")
    tags = ["Inventory"]

    # extract resource, function, endpoint_path, http verb
    plug = getattr(automation_engine.inventory, "assets")
    for cap in plug.capabilities:
        func = getattr(plug, cap)

        response_model = get_response_model(func)
        path = plug.path(cap)
        method = plug.http_verb(cap)
        router.add_api_route(
            f"/assets{path}",
            func,
            methods=[method],
            response_model=response_model,
            tags=tags
        )
    return router




from acex.configuration.components.base_component import ConfigComponent
from acex.configuration.components.interfaces import Interface
from acex_devkit.models.composed_configuration import (
    AuthorizedKey, 
    ReferenceTo,
    SshServer as SshServerAttributes
)

class SshServer(ConfigComponent):
    type = "ssh_server"
    model_cls = SshServerAttributes

    def pre_init(self):
        # Resolve source_interface
        if "source_interface" in self.kwargs:
            si = self.kwargs.pop("source_interface")
            if isinstance(si, type(None)):
                pass
            elif isinstance(si, str):
                ref = ReferenceTo(pointer=f"interfaces.{si}")
                self.kwargs["source_interface"] = ref

            elif isinstance(si, Interface):
                ref = ReferenceTo(pointer=f"interfaces.{si.name}")
                self.kwargs["source_interface"] = ref



class AuthorizedKey(ConfigComponent):
    type = "authorized_key"
    model_cls = AuthorizedKey



from .asset import Asset, AssetCluster, AssetClusterLink, AssetResponse, Ned
from .logical_node import LogicalNode, LogicalNodeResponse, LogicalNodeListResponse
from .node import Node, NodeResponse, NodeListResponse, NodeStatus
#from .external_value import ExternalValue
#from .attribute_value import AttributeValue
from acex_devkit.models import ExternalValue, AttributeValue
from .device_config import StoredDeviceConfig, DeviceConfig, DeviceConfigResponse
from .management_connections import ManagementConnection, ManagementConnectionResponse, ManagementConnectionBase
from .sites import Site, SiteResponse
from .contacts import Contact, ContactResponse, ContactAssignment
from .pagination import PaginatedResponse

system_models = [Asset, Ned, LogicalNode, Node, Site, Contact, ContactAssignment]


from .integration_plugin_base import IntegrationPluginBase
from .integration_plugin_factory_base import IntegrationPluginFactoryBase
from acex.utils import RestClient
from acex.models import AssetResponse, LogicalNodeResponse

import json
from requests import Response


# TODO: Fixa en egen hantering av response formatering. 
# Behöver få med:
# - DatasourceValue
# - Metadata
# - Modellering och validering av standardobjekt
# - Hämta interface uppmappade i netbox för assets? 


class Netbox(IntegrationPluginFactoryBase): 

    def __init__(self, url: str, token: str, verify_ssl: bool = True):
        """
        Used to define the connection to netbox. This is a factory class
        that will create a NetboxPluginInstance for each model used.
        """
        self.base_url = f"{url}api/"
        self.token = token
        self.rest = RestClient(self.base_url, verify_ssl=verify_ssl)
        self.rest.add_header("Authorization", f"Token {self.token}")
        self.rest.add_header("Content-Type", "application/json")


    def create_plugin(self, type: str|None = None) -> 'NetboxPlugin':
        """
        Create a plugin instance for a specific model.
        If no type is selected, generic plugin is returned.
        :param type: string representing the model/type to retreive.
        :return: An instance of NetboxPlugin.
        """
        if type is not None:
            return NetboxObjectPlugin(type, restclient=self.rest)
        else:
            return NetboxGenericPlugin(restclient=self.rest)


class NetboxGenericPlugin(IntegrationPluginBase):
    RESOURCE_TYPES = [
        "prefixes",
        "ip_addresses"
    ]

    DATA_TYPES= [
        "ip_addresses"
    ]

    def __init__(self, restclient: RestClient):
        self.rest = restclient
    
    def __repr__(self):
        return f"{self.__class__.__name__}:{self.rest.base_url}"

    def _type_to_endpoint(self, kind) -> str:
        """
        Maps the endpoint for a given object kind
        """
        match kind:
            case "ip_addresses":
                return "ipam/ip-addresses"

    def _get_output_from_response(self, kind, json_dict: dict):
        """
        Extract data from a json response based on object type.
        """
        match kind:
            case "ip_addresses":
                return json_dict["address"]

    def query(self, kind: str, query: dict): 
        endpoint = self._type_to_endpoint(kind)
        if "id" in query:
            endpoint = endpoint + f"/{query["id"]}"
        data = self.rest.get(endpoint, query).json()
        return self._get_output_from_response(kind, data)


class NetboxObjectPlugin(IntegrationPluginBase):
    """
    The actual plugin instance that will be used by the adapter in the
    inventory. Will be instanciated for each model used. Using the Netbox class
    to define the connection details.
    """
    PLUGIN_NAME = "netbox"
    OBJECT_TYPES = ["assets", "logical_nodes"]
    ENTITY_ENDPOINTS = {
        "assets": "dcim/devices/",
        "logical_nodes": "dcim/devices/",
    }
    OBJ_CLASS_MAP = {
        "assets": AssetResponse,
        "logical_nodes": LogicalNodeResponse
    }

    def __init__(self, type:str, restclient: RestClient):
        """
        Initialize the Netbox datasource plugin with a Netbox API client.
        :param url: The base URL of the Netbox instance.
        :param token: The API token for authentication.
        :param verify_ssl: Whether to verify SSL certificates (default is True).
        """
        self.rest = restclient
        self.type = type


    def _get_endpoint(self, id: str = None, filters: dict | None = None) -> str:
        """
        Return endpoint for the object type.
        """
        url =  self.__class__.ENTITY_ENDPOINTS.get(self.type, "")
        if id is not None:
            url += f"{id}/"
        elif filters:
            url += "?" + "&".join(f"{k}={v}" for k, v in filters.items())
        return url

    def _format_model(self, json_data: dict):
        model = self.__class__.OBJ_CLASS_MAP.get(self.type)
        return model(**json_data)


    def get(self, id: str):
        response = self.rest.get(self._get_endpoint(id=id))
        response = self._format_model(response.json())
        print(response.json())
        return response


    def query(self, filters: dict | None = None) -> list:
        response = []
        nb_response = self.rest.get(self._get_endpoint(filters=filters))
        for i in nb_response.json()["results"]:
            response.append(self._format_model(i))
        return response



import inspect
from typing import List, Optional

from sqlalchemy import select

from acex.models import Asset, AssetResponse, PaginatedResponse
from acex.models.node import Node, AssetRefType


class AssetService:
    """Service layer för Asset business logic."""

    def __init__(self, adapter):
        self.adapter = adapter

    async def _call_method(self, method, *args, **kwargs):
        """Helper för att hantera både sync och async metoder."""
        if inspect.iscoroutinefunction(method):
            return await method(*args, **kwargs)
        else:
            return method(*args, **kwargs)

    async def create(self, asset: Asset):
        result = await self._call_method(self.adapter.create, asset)
        return result

    async def get(self, id: str) -> AssetResponse:
        result = await self._call_method(self.adapter.get, id)
        return result

    async def query(
        self,
        vendor: str = None,
        os: str = None,
        hardware_model: str = None,
        ned_id: str = None,
        serial_number: str = None,
        assigned: Optional[bool] = None,
        limit: int = 100,
        offset: int = 0,
    ) -> PaginatedResponse[AssetResponse]:

        query_filters = {
            k: v for k, v in {
                "vendor": vendor,
                "os": os,
                "hardware_model": hardware_model,
                "serial_number": serial_number,
                "ned_id": ned_id,
            }.items() if v is not None
        }

        extra_filters = []
        if assigned is not None:
            assigned_ids = select(Node.asset_ref_id).where(
                Node.asset_ref_type == AssetRefType.asset
            )
            if assigned:
                extra_filters.append(Asset.id.in_(assigned_ids))
            else:
                extra_filters.append(~Asset.id.in_(assigned_ids))

        result = await self._call_method(
            self.adapter.query, filters=query_filters, extra_filters=extra_filters or None, limit=limit, offset=offset
        )
        return PaginatedResponse(items=result["items"], total=result["total"], limit=limit, offset=offset)

    async def update(self, id: str, asset: Asset):
        result = await self._call_method(self.adapter.update, id, asset)
        return result

    async def delete(self, id: str):
        result = await self._call_method(self.adapter.delete, id)
        return result

    @property
    def capabilities(self):
        return self.adapter.capabilities

    def path(self, capability):
        return self.adapter.path(capability)

    def http_verb(self, capability):
        return self.adapter.http_verb(capability)



import inspect
from acex.models.sites import Site, SiteBase, SiteResponse
from acex.models.pagination import PaginatedResponse
from typing import List


class SiteService:
    """Service layer för Site business logic."""

    def __init__(self, adapter, inventory=None):
        self.adapter = adapter
        self.inventory = inventory

    async def _call_method(self, method, *args, **kwargs):
        """Helper för att hantera både sync och async metoder."""
        if inspect.iscoroutinefunction(method):
            return await method(*args, **kwargs)
        else:
            return method(*args, **kwargs)

    async def _enrich_data(self, site):
        """Berika site med tillhörande kontakter via assignments."""
        if site is None:
            return None
        data = site.model_dump() if hasattr(site, 'model_dump') else dict(site)
        contacts = []
        if self.inventory and hasattr(self.inventory, 'contact_assignment_manager'):
            assignments = self.inventory.contact_assignment_manager.list_assignments(site_name=site.name)
            for a in assignments:
                result = await self.inventory.contacts.query(name=a.contact_name)
                if result.items:
                    contacts.append(result.items[0])
        data["contacts"] = contacts
        return SiteResponse(**data)

    async def create(self, site: SiteBase):
        result = await self._call_method(self.adapter.create, Site(**site.model_dump()))
        return result

    async def get(self, id: str) -> SiteResponse:
        result = await self._call_method(self.adapter.get, id)
        return await self._enrich_data(result)

    async def query(
        self,
        name: str = None,
        display_name: str = None,
        city: str = None,
        country: str = None,
        limit: int = 100,
        offset: int = 0,
    ) -> PaginatedResponse[SiteResponse]:

        query_filters = {
            k: v for k, v in {
                "name": name,
                "display_name": display_name,
                "city": city,
                "country": country,
            }.items() if v is not None
        }

        result = await self._call_method(self.adapter.query, filters=query_filters, limit=limit, offset=offset)
        items = [await self._enrich_data(s) for s in result["items"]]
        return PaginatedResponse(items=items, total=result["total"], limit=limit, offset=offset)

    async def update(self, id: str, site: Site):
        result = await self._call_method(self.adapter.update, id, site)
        return result

    async def delete(self, id: str):
        result = await self._call_method(self.adapter.delete, id)
        return result

    @property
    def capabilities(self):
        return self.adapter.capabilities

    def path(self, capability):
        return self.adapter.path(capability)

    def http_verb(self, capability):
        return self.adapter.http_verb(capability)



from typing import Dict, Type, Optional
from acex.plugins.integrations.integration_plugin_factory_base import IntegrationPluginFactoryBase, IntegrationPluginBase

class PluginManager:
    """
    Hanterar registrering och hämtning av både specifika och generiska plugins.
    """
    def __init__(self):
        # En factory per objekt-typ (men samma factory kan användas för flera typer)
        self._object_type_map: Dict[str, IntegrationPluginFactoryBase] = {}

        # En factory per generiskt namn
        self._generic_plugin_map: Dict[str, IntegrationPluginFactoryBase] = {}

    def register_type_plugin(self, object_type: str, factory: IntegrationPluginFactoryBase):
        """
        Registrera en factory för en specifik objekt-typ (t.ex. "asset").
        """
        self._object_type_map[object_type] = factory

    def register_generic_plugin(self, plugin_name: str, factory: IntegrationPluginFactoryBase):
        """
        Registrera en factory för ett generiskt namn (t.ex. "netbox").
        """
        self._generic_plugin_map[plugin_name] = factory.create_plugin()

    def get_plugin_for_object_type(self, obj_type: str) -> Optional[IntegrationPluginBase]:
        plugin = self._object_type_map.get(obj_type)
        return None

    def get_generic_plugin(self, plugin_name: str,) -> Optional[IntegrationPluginBase]:
        plugin = self._generic_plugin_map.get(plugin_name)
        return plugin





from acex.configuration.components.base_component import ConfigComponent
from acex_devkit.models import AttributeValue
from pydantic import BaseModel


class SingleAttributeString(BaseModel):
    value: AttributeValue[str]


class HostName(ConfigComponent):
    type = "hostname"
    model_cls = SingleAttributeString

class Contact(ConfigComponent):
    type = "contact"
    model_cls = SingleAttributeString

class Location(ConfigComponent):
    type = "location"
    model_cls = SingleAttributeString

class DomainName(ConfigComponent):
    type = "domain-name"
    model_cls = SingleAttributeString

class LoginBanner(ConfigComponent):
    type = "login-banner"
    model_cls = SingleAttributeString

class MotdBanner(ConfigComponent):
    type = "motd-banner"
    model_cls = SingleAttributeString


from acex.configuration.components.base_component import ConfigComponent
from acex.configuration.components.interfaces import Interface
from acex_devkit.models.composed_configuration import ReferenceTo
from acex_devkit.models.logging import (
    LoggingConfig as LoggingConfigAttributes,
    Console as ConsoleAttributes,
    RemoteServer as RemoteServerAttributes,
    LoggingEvent as LoggingEventAttributes,
    VtyLine as VtyLineAttributes,
    FileLogging as FileLoggingAttributes,

)

class LoggingConfig(ConfigComponent):
    type= 'logging_config'
    model_cls = LoggingConfigAttributes

class Console(ConfigComponent):
    type = 'console'
    model_cls = ConsoleAttributes

class VtyLine(ConfigComponent):
    type = 'vty_line'
    model_cls = VtyLineAttributes

class RemoteServer(ConfigComponent):
    type = 'remote_server'
    model_cls = RemoteServerAttributes

class LoggingEvent(ConfigComponent):
    type = 'logging_event'
    model_cls = LoggingEventAttributes

class FileLogging(ConfigComponent):
    type = 'file_logging'
    #model_cls = FileConfigAttributes
    model_cls = FileLoggingAttributes



from acex.configuration.components.base_component import ConfigComponent

from acex_devkit.models.composed_configuration import (
    CdpConfigAttributes,
)

class CdpConfig(ConfigComponent): 
    type = "cdp_config"
    model_cls = CdpConfigAttributes


import inspect
from datetime import datetime
from acex.models import Node, NodeResponse, NodeListResponse, PaginatedResponse
from acex.models.node import NodeStatus
from acex.plugins.neds.manager.ned_manager import NEDManager
from acex.models.asset import Asset
from typing import List, Optional

class NodeService:
    """Service layer för Node business logik."""

    def __init__(self, adapter, inventory):
        self.adapter = adapter
        self.inventory = inventory

    async def _call_method(self, method, *args, **kwargs):
        """Helper för att hantera både sync och async metoder."""
        if inspect.iscoroutinefunction(method):
            return await method(*args, **kwargs)
        else:
            return method(*args, **kwargs)

    async def _enrich_data(self, node):
        """När en specific node hämtas vill vi, 
        berika responsen med datat från refererade objekt."""

        if node is None:
            return None

        node = node.model_dump()
        asset = None
        if node.get("asset_ref_type") == "asset_cluster":
            asset = self.inventory.asset_cluster_manager.get_cluster(node["asset_ref_id"])
        else:
            asset = await self.inventory.assets.get(node["asset_ref_id"])
        if isinstance(asset, Asset):
            node["asset"] = asset.model_dump()
        else:
            node["asset"] = asset
        ln = await self.inventory.logical_nodes.get(node["logical_node_id"])
        if ln is not None:
            node["logical_node"] = ln.model_dump()

        return NodeResponse(**node)

    async def get_rendered_config(self, id: str):
        """
        Renderar konfigurationen för en nod instans.
        """
        ni = await self.get(id)
        ln = await self.inventory.logical_nodes.get(ni.logical_node_id)
        composed_config = ln.configuration

        asset = None
        if getattr(ni, "asset_ref_type", "asset") == "asset_cluster":
            asset = self.inventory.asset_cluster_manager.get_cluster(ni.asset_ref_id)
        else:
            asset = await self.inventory.assets.get(ni.asset_ref_id)

        ned_manager = NEDManager()
        ned = ned_manager.get_driver_instance(asset.ned_id)

        if ned is None:
            return "error: NED not found"
        try:
            config = ned.render(configuration=composed_config, asset=asset)
        except Exception as e:
            # Skriver ut felmeddelandet
            print("Fel:", e)
            # Skriver ut hela tracebacken
            import traceback
            traceback.print_exc()
            # Todo, printa ut mer info från vilken render och vad som pajjat
            # Fel: render trasig
            # Render XYZ trasig
            # Node ID X triggade fel
            # På den här filen kommer tracen
            # Här kommer traceback mannen:
            # tracen
            
            config = ""
        return config
    
    async def create(self, logical_node: Node):
        result = await self._call_method(self.adapter.create, logical_node)
        return result
    
    async def get(self, id: str) -> NodeResponse:
        result = await self._call_method(self.adapter.get, id)
        result = await self._enrich_data(result)
        return result

    async def query(
        self,
        site: str = None,
        hostname: str = None,
        logical_node_id: int = None,
        asset_ref_id: int = None,
        status: Optional[NodeStatus] = None,
        limit: int = 100,
        offset: int = 0,
    ) -> PaginatedResponse[NodeListResponse]:

        query_filters = {
            k: v for k, v in {
                "logical_node.site": site,
                "logical_node.hostname": hostname,
                "logical_node_id": logical_node_id,
                "asset_ref_id": asset_ref_id,
                "status": status,
            }.items() if v is not None
        }
        result = await self._call_method(self.adapter.query, filters=query_filters, limit=limit, offset=offset)

        # Bulk-fetch unique assets to avoid N+1
        asset_ids = {n.asset_ref_id for n in result["items"]}
        assets_by_id = {}
        for aid in asset_ids:
            asset = await self._call_method(self.inventory.assets.adapter.get, aid)
            if asset:
                assets_by_id[asset.id] = asset

        items = []
        for node in result["items"]:
            asset = assets_by_id.get(node.asset_ref_id)
            items.append(NodeListResponse(
                **node.model_dump(),
                hostname=node.logical_node.hostname if node.logical_node else None,
                site=node.logical_node.site if node.logical_node else None,
                vendor=asset.vendor if asset else None,
                os=asset.os if asset else None,
                ned_id=asset.ned_id if asset else None,
            ))

        return PaginatedResponse(items=items, total=result["total"], limit=limit, offset=offset)

    async def update(self, id: str, logical_node: Node):
        logical_node.updated_at = datetime.utcnow()
        result = await self._call_method(self.adapter.update, id, logical_node)
        return result
    
    async def delete(self, id: str):
        result = await self._call_method(self.adapter.delete, id)
        return result
    
    @property
    def capabilities(self):
        return self.adapter.capabilities

    def path(self, capability):
        return self.adapter.path(capability)
    
    def http_verb(self, capability):
        return self.adapter.http_verb(capability)




from fastapi import APIRouter
from acex.constants import BASE_URL
from acex.plugins.neds.manager import NEDManager
from fastapi.responses import FileResponse
from fastapi import HTTPException
from acex.models.ned import Ned
from typing import List
import os


nm = NEDManager()
#nm.download_and_install_neds_in_specs()
nm.load_drivers()


def list_neds():
    neds = nm.list_drivers()
    return neds

def get_ned(ned_id: str):
    ned = nm.get_driver_info(ned_id)
    return ned

def download_ned(filename: str):
    full_path = f"{nm.driver_dir}/{filename}"
    if not os.path.exists(full_path):
        raise HTTPException(status_code=404, detail="Driver not found")

    return FileResponse(full_path, filename=filename)

def create_router(automation_engine):
    router = APIRouter(prefix=f"{BASE_URL}/neds")
    tags = ["Inventory"]
    router.add_api_route(
        "/",
        list_neds,
        methods=["GET"],
        tags=tags,
        response_model=List[Ned]
    )
    router.add_api_route(
        "/{ned_id}",
        get_ned,
        methods=["GET"],
        tags=tags,
        response_model=Ned
    )
    router.add_api_route(
        "/download/{filename}",
        download_ned,
        methods=["GET"],
        tags=tags
    )

    return router





import inspect
import functools
from fastapi import APIRouter, Response
from fastapi.responses import PlainTextResponse
from typing import Callable, Any, Optional
from acex.config_diff import DiffLogicalNode

from acex.constants import BASE_URL


def hello():
    return "hello"

def create_router(automation_engine):
    router = APIRouter(prefix=f"{BASE_URL}/health")
    tags = ["Health"]
    dcm = automation_engine.device_config_manager
    differ = DiffLogicalNode(automation_engine.inventory, dcm)

    router.add_api_route(
        "/node_instance/{node_instance_id}",
        differ.compliance_check_node_instance,
        methods=["GET"],
        tags=tags
    )

    router.add_api_route(
        "/site/{site_name}",
        differ.compliance_check_site,
        methods=["GET"],
        tags=tags
    )

    return router







from acex_devkit.configdiffer.configdiffer import ConfigDiffer

class DiffLogicalNode: 

    def __init__(self, inventory, device_config_manager):
        self.inventory = inventory
        self.dcm = device_config_manager


    async def diff(self, node_instance_id: str):

        # Load actual node 
        node = await self.inventory.node_instances.get(node_instance_id)

        # Get desired composed config
        desired_config = node.logical_node.configuration

        # Get observed parsed composed config
        observed_config = await self.dcm.get_latest_config(node_instance_id, "parsed")

        # diff and return diff!
        differ = ConfigDiffer()
        diff = differ.diff(
            desired_config=desired_config,
            observed_config=observed_config
        )

        return diff

    async def compliance_check_node_instance(self, node_instance_id: str):
        diff = await self.diff(node_instance_id)
        diff_dump = diff.model_dump()

        diff_dump.pop('changed')
        diff_dump.pop('added')
        diff_dump.pop('removed')
        return diff_dump

    async def compliance_check_site(self, site_name: str):

        response = {
            "nodes": {},
        }
        nodes = await self.inventory.node_instances.query(site=site_name)

        total_desired = 0
        total_observed = 0
        total_compliant = 0

        for node in nodes.items:
            compliance = await self.compliance_check_node_instance(str(node.id))
            response["nodes"][node.id] = compliance
            total_desired += compliance["total_desired"]
            total_observed += compliance["total_observed"]
            total_compliant += compliance["compliant_count"]

        response["summary"] = {
            "total_desired": total_desired,
            "total_observed": total_observed,
            "compliant_count": total_compliant,
            "compliance_percentage": round(total_compliant / total_desired * 100, 2) if total_desired > 0 else 100.0,
        }

        return response